package runners_test

import (
	"errors"
	"testing"
	"time"

	"codeberg.org/gruf/go-runners"
)

func TestProcessor(t *testing.T) {
	var p runners.Processor

	// Create test error ptr.
	terr := errors.New("test")

	// Create sync channel.
	ch := make(chan struct{})

	go func() {
		if err := p.Process(func() error {
			// Signal started.
			ch <- struct{}{}

			// Block until next
			// instance started.
			<-ch

			// Return known err ptr.
			return terr
		}); err != terr {
			t.Errorf("unexpected error returned from processor: %v", err)
		}
	}()

	// Wait on start.
	<-ch

	const count = 10

	for i := 0; i < count; i++ {
		go func() {
			if err := p.Process(func() error {
				// Return unknown error ptr
				return errors.New("unknown error")
			}); err != terr {
				t.Errorf("unexpected error returned from processor: %v", err)
			}
		}()
	}

	// Give time for secondary
	// processor goroutines to block.
	time.Sleep(time.Second)

	// Release first instance.
	ch <- struct{}{}

	// Give time for all
	// secondary to return.
	time.Sleep(time.Second)

	if err := p.Process(func() error {
		return nil
	}); err != nil {
		t.Errorf("unexpected error returned from processor: %v", err)
	}
}
